import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Arrays;

import mgui.io.util.IoFunctions;
import mgui.numbers.MguiDouble;


public class Temp8 {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// Compile a list of GM, WM, and GM volumes from the cls_volumes.dat file of subject directories
		// Output to a tab-delimited text file
		
		File civet_dir=new File("/export02/data/adni/civet_validated");
		File output_file = new File("/export02/data/adni/metadata/cls_volumes.txt");
		
		String[] subjects = civet_dir.list(IoFunctions.getDirFilter());
		Arrays.sort(subjects);
		
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(output_file));
			writer.write("subject\tcsf_volume\tgm_volume\twm_volume");
			
			int success_count = 0, fail_count = 0;
			for (int i = 0; i < subjects.length; i++){
				String subject = subjects[i];
				File input_file = new File(civet_dir.getAbsolutePath() + File.separator + subject + File.separator + "classify" +
										   File.separator + "adni_" + subject + "_cls_volumes.dat");
				boolean failed = false;
				if (!input_file.exists()){
					System.out.println("No data found for subject '" + subject + "'..");
					failed=true;
				}else{
					double csf_volume = -1, gm_volume = -1, wm_volume = -1;
					try{
						BufferedReader reader = new BufferedReader(new FileReader(input_file));
						
						String line = reader.readLine();
						csf_volume = Double.valueOf(line.substring(8));
						line = reader.readLine();
						gm_volume = Double.valueOf(line.substring(8));
						line = reader.readLine();
						wm_volume = Double.valueOf(line.substring(8));
						
						reader.close();
						
					}catch (IOException ex){
						System.out.println("Could not read data for subject '" + subject + "'..");
						failed=true;
						}
					
					if (!failed){
						writer.write("\n" + subject + "\t" + MguiDouble.getString(csf_volume, 2, true) + "\t" + 
															 MguiDouble.getString(gm_volume, 2, true) + "\t" + 
															 MguiDouble.getString(wm_volume, 2, true));
					
						success_count++;
					}else{
						fail_count++;
						}
					
					}
				
				}
			
			writer.close();
			
			System.out.println("Done. " + success_count + " sucesses. " + fail_count + " failures.");
			System.out.println("Output written to " + output_file.getAbsolutePath() + ".");
			
		}catch (IOException ex){
			System.out.println("Could not write data to '" + output_file.getAbsolutePath() + "'..");
			}

		
		
		
	}

}