import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;



public class ValidationViewer extends JPanel  {

	ImagePanel image_panel = new ImagePanel();
	ImageList image_list; // = new ImageList();
	
	String img_type;
	JFrame parent_frame;
	String prefix = "";
	
	boolean updateValue = true;
	
	public static void main(String[] args){
		
		JFrame frame = new JFrame("Civet Validation Viewer");
		
		ValidationViewer viewer = new ValidationViewer(args[0], args[1], args[2], frame);
		frame.setContentPane(viewer);
		
		frame.pack();
		frame.setVisible(true);
	}
	
	public ValidationViewer(String subject_dir, String img_type, String prefix, JFrame parent){
		
		this.prefix = prefix;
		this.img_type = img_type;
		image_list = new ImageList();
		image_list.setSubjectDir(subject_dir);
		this.parent_frame = parent;
		parent_frame.setTitle("Minc Verification Viewer [" + img_type + "]");
		
		setLayout(new BorderLayout());
		image_panel.setBackground(Color.WHITE);
		image_panel.setBorder(BorderFactory.createLineBorder(Color.BLUE));
		
		this.add(image_panel, BorderLayout.CENTER);
		this.add(image_list, BorderLayout.WEST);
		
	}
	
	
	
	FilenameFilter getDirFilter(){
		return new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return new File(dir, name).isDirectory();
	        }
		};
	}
	
	class ImageList extends JPanel implements ListSelectionListener, ActionListener{
		DefaultListModel listModel = new DefaultListModel();
		ListSelectionModel selModel;
		JList subject_list = new JList(listModel);
		JScrollPane scroll_panel = new JScrollPane(subject_list);
		public String subject_dir;
		JComboBox cmb_image_type = new JComboBox();
		
		BufferedImage error_image;
		
		public ImageList(){
			cmb_image_type.addItem("clasp");
			cmb_image_type.addItem("verify");
			cmb_image_type.setSelectedItem(img_type);
			cmb_image_type.addActionListener(this);
			
			setLayout(new BorderLayout());
			add(cmb_image_type, BorderLayout.NORTH);
			add(scroll_panel, BorderLayout.CENTER);
			
			File file = new File("/export01/data/eclipse/workspace/sandbox/error_image.png");
			try{
				error_image = ImageIO.read(file);
			}catch (IOException e){
				e.printStackTrace();
				}
		}
		
		public void setSubjectDir(String s){
			subject_dir = s;
			
			selModel = subject_list.getSelectionModel();
			selModel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			selModel.addListSelectionListener(this);
			
//			load subject data
			ArrayList<String> subjects = getSubjectList(subject_dir); 
			
			if (subjects == null) return;
			for (int i = 0; i < subjects.size(); i++)
				listModel.addElement(subjects.get(i));
			
		}
		
		ArrayList<String> getSubjectList(String subject_dir){
			File dir = new File(subject_dir);
			
			if (!dir.isDirectory()){
				System.out.println(subject_dir + " is not a valid directory.");
				return null;
				}
			
			String[] subjects = dir.list(getDirFilter());
			Arrays.sort(subjects);
			
			ArrayList<String> list = new ArrayList<String>();
			for (int i = 0; i < subjects.length; i++)
				list.add(subjects[i]);
			
			return list;
			
		}
		
		void updateImage(){
			
			int i = subject_list.getSelectedIndex();
			if (i < 0) return;
			String sel = (String)listModel.getElementAt(i);
			if (sel == null) return;
			
			File f = new File(subject_dir + sel + File.separator + "verify" + File.separator + prefix + sel + "_" + img_type + ".png");
			try{
				BufferedImage bi = ImageIO.read(f);
				image_panel.setImage(bi);
				//parent_frame.setTitle("Minc Verification Viewer [" + img_type + "]: " + f.getName());
			}catch(IOException e){
				image_panel.setImage(error_image);
				
				//System.out.println("Error reading '" + f.getAbsolutePath() + "'");
				//e.printStackTrace();
				}
			parent_frame.setTitle("Minc Verification Viewer [" + img_type + "]: " + f.getName());
		}
		
		public void actionPerformed(ActionEvent e){
			
			if (cmb_image_type.getSelectedItem().equals(img_type)) return;
			img_type = (String)cmb_image_type.getSelectedItem();
			updateImage();
			
		}
		
		public void valueChanged(ListSelectionEvent e) {
			
			if (e.getValueIsAdjusting()) return;
			updateImage();
				
		}
		
	}
	
	class ImagePanel extends JPanel{
		
		Image current_image;
		
		public void setImage(Image image){
			current_image = image;
			updateUI();
		}
		
		public void paintComponent(Graphics g){
			super.paintComponent(g);
			Graphics2D g2 = (Graphics2D)g;
			
			if (current_image == null) return;
			
			int w = getWidth();
			int h = getHeight();
			
			g2.drawImage(current_image, 0, 0, w, h, null);
		}
		
	}
	
}
